function S = CoreTensors(A,U,V,W,R,Rank,prodI,prodR)
% S = CoreTensors(A,U,V,W,R,Rank,prodI,prodR) returns the optimal core tensors for the given factor matrices.
% INPUT: 
%  - A is the third-order tensor to be approximated by BTD;
%  - U is a third-order tensor containing the factor matrices U(:,:,1),...,U(:,:,R);
%  - V is a third-order tensor containing the factor matrices V(:,:,1),...,V(:,:,R);
%  - W is a third-order tensor containing the factor matrices W(:,:,1),...,W(:,:,R).
%  - R is the number of terms in the BTD;
%  - Rank is the multilinear rank of each term;
%  - prodI is the number of elements in A;
%  - prodR is the product of the elements of Rank.
% OUTPUT:
%  - S is a fourth-order tensor containing the R optimal core tensors S(:,:,:,1),...,S(:,:,:,R).
%% Least squares problem
a = tens2vec(A,[3,2,1]);
m = prodI; % number of rows in P
n = prodR*R; % number of columns in P
P = zeros(m,n);
for r = 1:R
    P(:,(r-1)*prodR+1:r*prodR) = kron(kron(U(:,:,r),V(:,:,r)),W(:,:,r));
end
s = P \ a;
%% Core tensors
S = zeros(Rank(1),Rank(2),Rank(3),R);
for r = 1:R
    S(:,:,:,r) = vec2tens(s((r-1)*prodR+1:r*prodR),Rank,[3,2,1]);
end
end