function obj = f(A,I,R,S,U,V,W)
% obj = f(A,I,R,S,U,V,W) evaluates the cost function f_A.
% INPUT: 
%  - A is the third-order tensor to be approximated by BTD;
%  - I is the size of A;
%  - R is the number of terms in the BTD;
%  - S is a fourth-order tensor containing the R core tensors;
%  - U is a third-order tensor containing the factor matrices U(:,:,1),...,U(:,:,R);
%  - V is a third-order tensor containing the factor matrices V(:,:,1),...,V(:,:,R);
%  - W is a third-order tensor containing the factor matrices W(:,:,1),...,W(:,:,R).
% OUTPUT:
%  - obj is the value of the objective function f_A.
B = zeros(I);
for r = 1:R
    B = B + tmprod(S(:,:,:,r),{U(:,:,r),V(:,:,r),W(:,:,r)},[1,2,3]);
end
E = B-A;
obj = inprod(E,E);
end